@@ -4,10 +4,11 @@ from __future__ import annotations
 
 import json
 import re
+from pathlib import Path
 
-from .utils import GITHUB_API_URL, MAX_PROMPT_CHARS, Action, get_completion, remove_html_comments
+from .utils import ACTIONS_CREDIT, GITHUB_API_URL, MAX_PROMPT_CHARS, Action, get_response, remove_html_comments
 
-REVIEW_MARKER = "🔍 PR Review"
+REVIEW_MARKER = "## 🔍 PR Review"
 ERROR_MARKER = "⚠️ Review generation encountered an error"
 EMOJI_MAP = {"CRITICAL": "❗", "HIGH": "⚠️", "MEDIUM": "💡", "LOW": "📝", "SUGGESTION": "💭"}
 SKIP_PATTERNS = [
@@ -30,9 +31,10 @@ SKIP_PATTERNS = [
 ]
 
 
-def parse_diff_files(diff_text: str) -> dict:
-    """Parse diff to extract file paths, valid line numbers, and line content for comments (both sides)."""
+def parse_diff_files(diff_text: str) -> tuple[dict, str]:
+    """Parse diff and return file mapping with line numbers AND augmented diff with explicit line numbers."""
     files, current_file, new_line, old_line = {}, None, 0, 0
+    augmented_lines = []
 
     for line in diff_text.split("\n"):
         if line.startswith("diff --git"):
@@ -41,31 +43,41 @@ def parse_diff_files(diff_text: str) -> dict:
             new_line, old_line = 0, 0
             if current_file:
                 files[current_file] = {"RIGHT": {}, "LEFT": {}}
+            augmented_lines.append(line)
         elif line.startswith("@@") and current_file:
-            # Extract both old and new line numbers
             match = re.search(r"@@ -(\d+)(?:,\d+)? \+(\d+)(?:,\d+)?", line)
             if match:
                 old_line, new_line = int(match.group(1)), int(match.group(2))
+            augmented_lines.append(line)
         elif current_file and (new_line > 0 or old_line > 0):
             if line.startswith("+") and not line.startswith("+++"):
-                files[current_file]["RIGHT"][new_line] = line[1:]  # Added line (right/new side)
+                files[current_file]["RIGHT"][new_line] = line[1:]
+                augmented_lines.append(f"R{new_line:>5} {line}")  # Prefix with RIGHT line number
                 new_line += 1
             elif line.startswith("-") and not line.startswith("---"):
-                files[current_file]["LEFT"][old_line] = line[1:]  # Removed line (left/old side)
+                files[current_file]["LEFT"][old_line] = line[1:]
+                augmented_lines.append(f"L{old_line:>5} {line}")  # Prefix with LEFT line number
                 old_line += 1
-            elif not line.startswith("\\"):  # Context line (ignore "No newline" markers)
+            elif not line.startswith("\\"):
+                augmented_lines.append(f"       {line}")  # Context line, no number
                 new_line += 1
                 old_line += 1
+            else:
+                augmented_lines.append(line)
+        else:
+            augmented_lines.append(line)
 
-    return files
+    return files, "\n".join(augmented_lines)
 
 
-def generate_pr_review(repository: str, diff_text: str, pr_title: str, pr_description: str) -> dict:
+def generate_pr_review(
+    repository: str, diff_text: str, pr_title: str, pr_description: str, event: Action = None
+) -> dict:
     """Generate comprehensive PR review with line-specific comments and overall assessment."""
     if not diff_text:
         return {"comments": [], "summary": "No changes detected in diff"}
 
-    diff_files = parse_diff_files(diff_text)
+    diff_files, augmented_diff = parse_diff_files(diff_text)
     if not diff_files:
         return {"comments": [], "summary": "No files with changes detected in diff"}
 
@@ -82,45 +94,68 @@ def generate_pr_review(repository: str, diff_text: str, pr_title: str, pr_descri
         return {"comments": [], "summary": f"All {skipped_count} changed files are generated/vendored (skipped review)"}
 
     file_list = list(diff_files.keys())
-    diff_truncated = len(diff_text) > MAX_PROMPT_CHARS
+    diff_truncated = len(augmented_diff) > MAX_PROMPT_CHARS
     lines_changed = sum(len(sides["RIGHT"]) + len(sides["LEFT"]) for sides in diff_files.values())
 
+    # Fetch full file contents for better context if within token budget
+    full_files_section = ""
+    if event and len(file_list) <= 10:  # Reasonable file count limit
+        file_contents = []
+        total_chars = len(augmented_diff)
+        for file_path in file_list:
+            try:
+                local_path = Path(file_path)
+                if not local_path.exists():
+                    continue
+                content = local_path.read_text(encoding="utf-8")
+                # Only include if within budget
+                if total_chars + len(content) + 1000 < MAX_PROMPT_CHARS:  # 1000 char buffer for formatting
+                    file_contents.append(f"### {file_path}\n```\n{content}\n```")
+                    total_chars += len(content) + 1000
+                else:
+                    break  # Stop when we hit budget limit
+            except Exception:
+                continue
+        if file_contents:
+            full_files_section = f"FULL FILE CONTENTS:\n{chr(10).join(file_contents)}\n\n"
+
     content = (
-        "You are an expert code reviewer for Ultralytics. Provide detailed inline comments on specific code changes.\n\n"
-        "Focus on: Bugs, security, performance, best practices, edge cases, error handling\n\n"
-        "FORMATTING: Use backticks for code: `x=3`, `file.py`, `function()`\n\n"
+        "You are an expert code reviewer for Ultralytics. Review the code changes and provide inline comments where you identify issues or opportunities for improvement.\n\n"
+        "Focus on: bugs, security vulnerabilities, performance issues, best practices, edge cases, error handling, and code clarity.\n\n"
         "CRITICAL RULES:\n"
-        "1. Quality over quantity - zero comments is fine for clean code, only flag truly important issues\n"
-        "2. Combine issues that are directly related to the same problem\n"
-        "3. Use 'start_line' and 'line' to highlight multi-line ranges when issues span multiple lines\n"
+        "1. Provide balanced, constructive feedback - flag bugs, improvements, and best practice issues\n"
+        "2. For issues spanning multiple adjacent lines, use 'start_line' to create ONE multi-line comment, never separate comments\n"
+        "3. Combine related issues into a single comment when they stem from the same root cause\n"
         "4. Prioritize: CRITICAL bugs/security > HIGH impact > code quality improvements\n"
         "5. Keep comments concise and friendly - avoid jargon\n"
-        "6. Skip routine changes: imports, version updates, standard refactoring\n\n"
+        "6. Use backticks for code: `x=3`, `file.py`, `function()`\n"
+        "7. Skip routine changes: imports, version updates, standard refactoring\n\n"
         "SUMMARY:\n"
         "- Brief and actionable - what needs fixing, not where (locations shown in inline comments)\n\n"
         "SUGGESTIONS:\n"
-        "- ONLY provide 'suggestion' field when you have high certainty the code is problematic AND sufficient context for a confident fix\n"
-        "- If uncertain about the correct fix, omit 'suggestion' field and explain the concern in 'message' only\n"
-        "- Suggestions must be ready-to-merge code with NO comments, placeholders, or explanations\n"
-        "- Suggestions replace ONLY the single line at 'line' - for multi-line fixes, describe the change in 'message' instead\n"
-        "- Do NOT provide 'start_line' when including a 'suggestion' - suggestions are always single-line only\n"
-        "- Suggestion content must match the exact indentation of the original line\n"
-        "- Avoid triple backticks (```) in suggestions as they break markdown formatting\n"
-        "- It's better to flag an issue without a suggestion than provide a wrong or uncertain fix\n\n"
+        "- Provide 'suggestion' field with ready-to-merge code when you can confidently fix the issue\n"
+        "- Suggestions must be complete, working code with NO comments, placeholders, or explanations\n"
+        "- For single-line fixes: provide 'suggestion' without 'start_line' to replace the line at 'line'\n"
+        "- Do not provide multi-line fixes: suggestions should only be single line\n"
+        "- Match the exact indentation of the original code\n"
+        "- Web search is available to consult docs, dependencies, or technical details\n"
+        "- Avoid triple backticks (```) in suggestions as they break markdown formatting\n\n"
         "LINE NUMBERS:\n"
-        "- You MUST extract line numbers directly from the @@ hunk headers in the diff below\n"
-        "- RIGHT (added +): Find @@ lines, use numbers after +N (e.g., @@ -10,5 +20,7 @@ means RIGHT starts at line 20)\n"
-        "- LEFT (removed -): Find @@ lines, use numbers after -N (e.g., @@ -10,5 +20,7 @@ means LEFT starts at line 10)\n"
-        "- Count forward from hunk start: + lines increment RIGHT, - lines increment LEFT, context lines increment both\n"
-        "- CRITICAL: Using line numbers not in the diff will cause your comment to be rejected\n"
-        "- Suggestions only work on RIGHT (added) lines, never on LEFT (removed) lines\n\n"
+        "- Each line in the diff is prefixed with its line number for clarity:\n"
+        "  R  123 +added code     <- RIGHT side (new file), line 123\n"
+        "  L   45 -removed code   <- LEFT side (old file), line 45\n"
+        "         context line    <- context (no number needed)\n"
+        "- Extract the number after R or L prefix to get the exact line number\n"
+        "- Use 'side': 'RIGHT' for R-prefixed lines, 'side': 'LEFT' for L-prefixed lines\n"
+        "- Suggestions only work on RIGHT lines, never on LEFT lines\n"
+        "- CRITICAL: Only use line numbers that you see explicitly prefixed in the diff\n\n"
         "Return JSON: "
         '{"comments": [{"file": "exact/path", "line": N, "side": "RIGHT", "severity": "HIGH", "message": "..."}], "summary": "..."}\n\n'
         "Rules:\n"
-        "- Verify line numbers from @@ hunks: +N for RIGHT (added), -N for LEFT (removed)\n"
-        "- Exact paths (no ./), 'side' field defaults to RIGHT if omitted\n"
+        "- Extract line numbers from R#### or L#### prefixes in the diff\n"
+        "- Exact paths (no ./), 'side' field must match R (RIGHT) or L (LEFT) prefix\n"
         "- Severity: CRITICAL, HIGH, MEDIUM, LOW, SUGGESTION\n"
-        f"- Files changed: {len(file_list)} ({', '.join(file_list[:10])}{'...' if len(file_list) > 10 else ''})\n"
+        f"- Files changed: {len(file_list)} ({', '.join(file_list[:30])}{'...' if len(file_list) > 30 else ''})\n"
         f"- Lines changed: {lines_changed}\n"
     )
 
@@ -129,26 +164,75 @@ def generate_pr_review(repository: str, diff_text: str, pr_title: str, pr_descri
         {
             "role": "user",
             "content": (
-                f"Review this PR in https://github.com/{repository}:\n"
-                f"Title: {pr_title}\n"
-                f"Description: {remove_html_comments(pr_description or '')[:1000]}\n\n"
-                f"Diff:\n{diff_text[:MAX_PROMPT_CHARS]}\n\n"
+                f"Review this PR in https://github.com/{repository}:\n\n"
+                f"TITLE:\n{pr_title}\n\n"
+                f"BODY:\n{remove_html_comments(pr_description or '')[:1000]}\n\n"
+                f"{full_files_section}"
+                f"DIFF:\n{augmented_diff[:MAX_PROMPT_CHARS]}\n\n"
                 "Now review this diff according to the rules above. Return JSON with comments array and summary."
             ),
         },
     ]
 
-    try:
-        response = get_completion(messages, reasoning_effort="medium", model="gpt-5-codex")
+    # Debug output
+    # print(f"\nSystem prompt (first 3000 chars):\n{messages[0]['content'][:3000]}...\n")
+    # print(f"\nUser prompt (first 3000 chars):\n{messages[1]['content'][:3000]}...\n")
 
-        json_str = re.search(r"```(?:json)?\s*(\{.*?\})\s*```", response, re.DOTALL)
-        review_data = json.loads(json_str.group(1) if json_str else response)
-        print(json.dumps(review_data, indent=2))
-        print(f"AI generated {len(review_data.get('comments', []))} comments")
+    try:
+        schema = {
+            "type": "object",
+            "properties": {
+                "comments": {
+                    "type": "array",
+                    "items": {
+                        "type": "object",
+                        "properties": {
+                            "file": {"type": "string"},
+                            "line": {"type": "integer"},
+                            "side": {"type": "string", "enum": ["LEFT", "RIGHT"]},
+                            "severity": {"type": "string", "enum": ["CRITICAL", "HIGH", "MEDIUM", "LOW", "SUGGESTION"]},
+                            "message": {"type": "string"},
+                            "start_line": {"type": ["integer", "null"]},
+                            "suggestion": {"type": ["string", "null"]},
+                        },
+                        "required": ["file", "line", "side", "severity", "message", "start_line", "suggestion"],
+                        "additionalProperties": False,
+                    },
+                },
+                "summary": {"type": "string"},
+            },
+            "required": ["comments", "summary"],
+            "additionalProperties": False,
+        }
+
+        response = get_response(
+            messages,
+            reasoning_effort="low",
+            model="gpt-5-codex",
+            text_format={"format": {"type": "json_schema", "name": "pr_review", "strict": True, "schema": schema}},
+            tools=[
+                {
+                    "type": "web_search",
+                    "filters": {
+                        "allowed_domains": [
+                            "ultralytics.com",
+                            "github.com",
+                            "stackoverflow.com",
+                        ]
+                    },
+                }
+            ],
+        )
+
+        print(json.dumps(response, indent=2))
+
+        # Count comments BEFORE filtering (for COMMENT vs APPROVE decision)
+        comments_before_filtering = len(response.get("comments", []))
+        print(f"AI generated {comments_before_filtering} comments")
 
         # Validate, filter, and deduplicate comments
         unique_comments = {}
-        for c in review_data.get("comments", []):
+        for c in response.get("comments", []):
             file_path, line_num = c.get("file"), c.get("line", 0)
             start_line = c.get("start_line")
             side = (c.get("side") or "RIGHT").upper()  # Default to RIGHT (added lines)
@@ -158,20 +242,14 @@ def generate_pr_review(repository: str, diff_text: str, pr_title: str, pr_descri
                 print(f"Filtered out {file_path}:{line_num} (file not in diff)")
                 continue
             if line_num not in diff_files[file_path].get(side, {}):
-                # Try other side if not found
-                other_side = "LEFT" if side == "RIGHT" else "RIGHT"
-                if line_num in diff_files[file_path].get(other_side, {}):
-                    print(f"Switching {file_path}:{line_num} from {side} to {other_side}")
-                    c["side"] = other_side
-                    side = other_side
-                    # GitHub rejects suggestions on removed lines
-                    if side == "LEFT" and c.get("suggestion"):
-                        print(f"Dropping suggestion for {file_path}:{line_num} - LEFT side doesn't support suggestions")
-                        c.pop("suggestion", None)
-                else:
-                    available = {s: list(diff_files[file_path][s].keys())[:10] for s in ["RIGHT", "LEFT"]}
-                    print(f"Filtered out {file_path}:{line_num} (available: {available})")
-                    continue
+                available = {s: list(diff_files[file_path][s].keys())[:10] for s in ["RIGHT", "LEFT"]}
+                print(f"Filtered out {file_path}:{line_num} (side={side}, available: {available})")
+                continue
+
+            # GitHub rejects suggestions on removed lines
+            if side == "LEFT" and c.get("suggestion"):
+                print(f"Dropping suggestion for {file_path}:{line_num} - LEFT side doesn't support suggestions")
+                c.pop("suggestion", None)
 
             # Validate start_line if provided - drop start_line for suggestions (single-line only)
             if start_line:
@@ -192,16 +270,17 @@ def generate_pr_review(repository: str, diff_text: str, pr_title: str, pr_descri
             else:
                 print(f"⚠️  AI duplicate for {key}: {c.get('severity')} - {(c.get('message') or '')[:60]}...")
 
-        review_data.update(
+        response.update(
             {
                 "comments": list(unique_comments.values()),
+                "comments_before_filtering": comments_before_filtering,
                 "diff_files": diff_files,
                 "diff_truncated": diff_truncated,
                 "skipped_files": skipped_count,
             }
         )
-        print(f"Valid comments after filtering: {len(review_data['comments'])}")
-        return review_data
+        print(f"Valid comments after filtering: {len(response['comments'])}")
+        return response
 
     except Exception as e:
         import traceback
@@ -251,32 +330,16 @@ def post_review_summary(event: Action, review_data: dict, review_number: int) ->
     comments = review_data.get("comments", [])
     summary = review_data.get("summary") or ""
 
-    # Don't approve if error occurred or if there are critical/high severity issues
+    # Don't approve if error occurred, inline comments exist, or critical/high severity issues
     has_error = not summary or ERROR_MARKER in summary
+    has_inline_comments = review_data.get("comments_before_filtering", 0) > 0
     has_issues = any(c.get("severity") not in ["LOW", "SUGGESTION", None] for c in comments)
-    requests_changes = any(
-        phrase in summary.lower()
-        for phrase in [
-            "please",
-            "should",
-            "must",
-            "raise",
-            "needs",
-            "before merging",
-            "fix",
-            "error",
-            "issue",
-            "problem",
-            "warning",
-            "concern",
-        ]
-    )
-    event_type = "COMMENT" if (has_error or has_issues or requests_changes) else "APPROVE"
+    event_type = "COMMENT" if (has_error or has_inline_comments or has_issues) else "APPROVE"
 
     body = (
-        f"## {review_title}\n\n"
-        "<sub>Made with ❤️ by [Ultralytics Actions](https://github.com/ultralytics/actions)</sub>\n\n"
-        f"{review_data.get('summary', 'Review completed')[:1000]}\n\n"  # Clip summary length
+        f"{review_title}\n\n"
+        f"{ACTIONS_CREDIT}\n\n"
+        f"{review_data.get('summary', 'Review completed')[:3000]}\n\n"  # Clip summary length
     )
 
     if comments:
@@ -296,13 +359,11 @@ def post_review_summary(event: Action, review_data: dict, review_number: int) ->
             continue
 
         severity = comment.get("severity") or "SUGGESTION"
-        comment_body = f"{EMOJI_MAP.get(severity, '💭')} **{severity}**: {(comment.get('message') or '')[:1000]}"
-
-        # Get side (LEFT for removed lines, RIGHT for added lines)
         side = comment.get("side", "RIGHT")
+        comment_body = f"{EMOJI_MAP.get(severity, '💭')} **{severity}**: {(comment.get('message') or '')[:3000]}"
 
         if suggestion := comment.get("suggestion"):
-            suggestion = suggestion[:1000]  # Clip suggestion length
+            suggestion = suggestion[:3000]  # Clip suggestion length
             if "```" not in suggestion:
                 # Extract original line indentation and apply to suggestion
                 if original_line := review_data.get("diff_files", {}).get(file_path, {}).get(side, {}).get(line):
@@ -312,19 +373,16 @@ def post_review_summary(event: Action, review_data: dict, review_number: int) ->
 
         # Build comment with optional start_line for multi-line context
         review_comment = {"path": file_path, "line": line, "body": comment_body, "side": side}
-        if start_line := comment.get("start_line"):
-            if start_line < line:
-                review_comment["start_line"] = start_line
-                review_comment["start_side"] = side
-                print(f"Multi-line comment: {file_path}:{start_line}-{line} ({side})")
+        if (start_line := comment.get("start_line")) and start_line < line:
+            review_comment["start_line"] = start_line
+            review_comment["start_side"] = side
 
         review_comments.append(review_comment)
 
     # Submit review with inline comments
-    payload = {"commit_id": commit_sha, "body": body, "event": event_type}
+    payload = {"commit_id": commit_sha, "body": body.strip(), "event": event_type}
     if review_comments:
         payload["comments"] = review_comments
-        print(f"Posting review with {len(review_comments)} inline comments")
 
     event.post(
         f"{GITHUB_API_URL}/repos/{event.repository}/pulls/{pr_number}/reviews",
@@ -354,7 +412,7 @@ def main(*args, **kwargs):
     review_number = dismiss_previous_reviews(event)
 
     diff = event.get_pr_diff()
-    review = generate_pr_review(event.repository, diff, event.pr.get("title") or "", event.pr.get("body") or "")
+    review = generate_pr_review(event.repository, diff, event.pr.get("title") or "", event.pr.get("body") or "", event)
 
     post_review_summary(event, review, review_number)
     print("PR review completed")
