@@ -1,6 +1,7 @@
 # Ultralytics 🚀 AGPL-3.0 License - https://ultralytics.com/license
 
 from .common_utils import (
+    ACTIONS_CREDIT,
     REDIRECT_END_IGNORE_LIST,
     REDIRECT_START_IGNORE_LIST,
     REQUESTS_HEADERS,
@@ -12,30 +13,31 @@ from .github_utils import GITHUB_API_URL, GITHUB_GRAPHQL_URL, Action, ultralytic
 from .openai_utils import (
     MAX_PROMPT_CHARS,
     filter_labels,
-    get_completion,
     get_pr_open_response,
     get_pr_summary_guidelines,
     get_pr_summary_prompt,
+    get_response,
 )
 from .version_utils import check_pubdev_version, check_pypi_version
 
 __all__ = (
+    "ACTIONS_CREDIT",
     "GITHUB_API_URL",
     "GITHUB_GRAPHQL_URL",
     "MAX_PROMPT_CHARS",
+    "REDIRECT_END_IGNORE_LIST",
+    "REDIRECT_START_IGNORE_LIST",
     "REQUESTS_HEADERS",
     "URL_IGNORE_LIST",
-    "REDIRECT_START_IGNORE_LIST",
-    "REDIRECT_END_IGNORE_LIST",
     "Action",
     "allow_redirect",
     "check_pubdev_version",
     "check_pypi_version",
     "filter_labels",
-    "get_completion",
     "get_pr_open_response",
     "get_pr_summary_guidelines",
     "get_pr_summary_prompt",
+    "get_response",
     "remove_html_comments",
     "ultralytics_actions_info",
 )
